@@ -31,10 +31,6 @@ gettext.bindtextdomain('gpixpod', 'po')
 gettext.textdomain('gpixpod')
 gettext.install('gpixpod', 'po', True)
 gtk.glade.bindtextdomain('gpixpod', 'po')
-try:
-    from ipodhal import *
-except ImportError:
-    print _("iPod HAL autodetection disabled")
 
 
 class GPixPod:
@@ -49,7 +45,7 @@ class GPixPod:
 	    os.mkdir(self.preferencesdir)
 	self.preferencesfile = os.path.join(self.preferencesdir, 'config')
 	# Default preferences
-	self.prefs = self.defaultprefs = {'ipod_model':None, 'ipod_autodetect':True, 'ipod_mountpoint':"/mnt/ipod", 'ipod_askbeforeopen':True, 'photo_copyfullres':True,
+	self.prefs = self.defaultprefs = {'ipod_model':None, 'ipod_autodetect':False, 'ipod_mountpoint':"/mnt/ipod", 'ipod_askbeforeopen':True, 'photo_copyfullres':True,
 	                                  'path_lastphotodb':self.homedir, 'path_lastimages':self.homedir, 'path_lastsavedimages':self.homedir,
                                           'path_thumbscache':os.path.join(self.preferencesdir, '.SingleThumbs'), 'photo_behaviour':'Fit', 'photo_autorotate':'No',
                                           'gui_toolbar':True, 'gui_treeviewpreviews':False, 'gui_detailspane':True}
@@ -90,10 +86,24 @@ class GPixPod:
 			      'on_expander3_activate':(self.ExpandThumbnail, 2), 'on_expander4_activate':(self.ExpandThumbnail, 3),
 			      'on_save1_activate':self.Save, 'on_savebutton_clicked':self.Save,
 			      'on_treeview1_unselect_all':self.HideActions,
+                              'on_treeview1_button_press_event':self.TreeviewClick,
 			      #'on_window1_state_changed':self.Opening,
 			      'on_spinbutton1_value_changed':self.ZoomImage,
 			      'on_saveimagebutton_clicked':(self.ShowChooser, self.SaveImage, _('Images'), 'PIXBUF', False, self.prefs['path_lastsavedimages'], 
 			                                    False, gtk.FILE_CHOOSER_ACTION_SAVE, _('Save image')),
+                              'on_save1thumb_activate':(self.SaveThumb, 0),
+                              'on_save2thumb_activate':(self.SaveThumb, 1),
+                              'on_save3thumb_activate':(self.SaveThumb, 2),
+                              'on_save4thumb_activate':(self.SaveThumb, 3),
+                              'on_savefullres_activate':(self.ShowChooser, self.SaveImage, _('Images'), 'PIXBUF', False, self.prefs['path_lastsavedimages'],
+                                                         False, gtk.FILE_CHOOSER_ACTION_SAVE, _('Save image')),
+                              'on_showfullscreen_activate':self.ShowFullScreen,
+                              'on_startslideshow3_activate':(self.ShowFullScreen, True, 3),
+                              'on_startslideshow5_activate':(self.ShowFullScreen, True, 5),
+                              'on_startslideshow10_activate':(self.ShowFullScreen, True, 10),
+                              'on_startslideshow15_activate':(self.ShowFullScreen, True, 15),
+                              'on_startslideshow20_activate':(self.ShowFullScreen, True, 20),
+                              'on_startslideshow30_activate':(self.ShowFullScreen, True, 30),
                               'on_new_album_from_unreferenced_photos1_activate':self.NewAlbumFromUnreferencedPhotos,
 			      'on_cleancache1_activate':self.CleanCache, 'on_rescue_database1_activate':self.Rescue,
                               'on_progresspausebutton_clicked':self.PauseProgress, 'on_progressstopbutton_clicked':self.StopProgress,
@@ -164,7 +174,7 @@ class GPixPod:
         self.references_album_to_delete = []  # the entry is the album ID
         self.references_photo_to_delete = []  # the entry is a tuple of 2 elements: a photo ID and an album ID
 	# Setting sensitiveness
-	self.SetSensitive(('save1', 'savebutton', 'addalbum1', 'addalbumbutton', 'rename_album1', 'eject1', 'ejectbutton', 'ipodinfo1', 'spinbutton1', 'saveimagebutton', 'add_full_resolution', 'remove_full_resolution', 'new_album_from_unreferenced_photos1', 'list_previews1', 'expander1', 'expander2', 'expander3', 'expander4', 'cleancache1', 'rescue_database1'), False)
+	self.SetSensitive(('save1', 'savebutton', 'addalbum1', 'addalbumbutton', 'rename_album1', 'eject1', 'ejectbutton', 'ipodinfo1', 'spinbutton1', 'saveimagebutton', 'add_full_resolution', 'remove_full_resolution', 'new_album_from_unreferenced_photos1', 'list_previews1', 'expander1', 'expander2', 'expander3', 'expander4', 'cleancache1', 'rescue_database1', 'savefullres', 'save1thumb', 'save2thumb', 'save3thumb', 'save4thumb'), False)
 	self.HideActions(None)  # hide "Add Photo" and "Delete" by default
         self.tosave = False
 	self.DB = None  # Used to check if a Photo Database is loaded
@@ -175,7 +185,8 @@ class GPixPod:
 	if os.name == 'nt' and not os.path.isfile('DONATED'):
             self.DonateForGPixPod()
             gobject.timeout_add(300000, self.DonateForGPixPod)
-	if sys.modules.has_key('ipodhal') and self.prefs['ipod_autodetect']:  # If ipodhal.py has been imported successfully (python-dbus/hal/gnomevfs are available),
+	#if sys.modules.has_key('ipodhal') and self.prefs['ipod_autodetect']:  # If ipodhal.py has been imported successfully (python-dbus/hal/gnomevfs are available),
+        if self.prefs['ipod_autodetect']:
 	    gobject.idle_add(self.HALdetect)  # Checking in a loop events of connection and removal of an iPod
         else:
 	    self.Opening(None)
@@ -191,16 +202,25 @@ class GPixPod:
 
     def HALdetect(self):
         """ Detect using HAL """
-	if sys.modules.has_key('ipodhal') and self.prefs['ipod_autodetect']:
-	    #ipodHal will connect to HAL via DBUS
-	    #and 2 signal are implemented to detect ipod
-	    ipod_hal = iPodHal()
-	    #Connect to some signal
-	    ipod_hal.connect("ipod-added", self.OnIpodConnect)
-	    ipod_hal.connect("ipod-removed", self.OnIpodDisconnect)
-	    #Launch the detection 
-	    #if ipod already present signal 'ipod-added' is emit now
-	    ipod_hal.start_monitor()
+	if self.prefs['ipod_autodetect']:
+            try:
+                if 'ipodhal' not in sys.modules:
+                    import ipodhal
+	        #ipodHal will connect to HAL via DBUS
+	        #and 2 signal are implemented to detect ipod
+	        ipod_hal = ipodhal.iPodHal()
+	        #Connect to some signal
+	        ipod_hal.connect("ipod-added", self.OnIpodConnect)
+	        ipod_hal.connect("ipod-removed", self.OnIpodDisconnect)
+	        #Launch the detection 
+	        #if ipod already present signal 'ipod-added' is emit now
+	        ipod_hal.start_monitor()
+                return True
+            except ImportError:
+                print _("iPod HAL autodetection disabled")
+                return False
+        else:
+            return False
 
     def Opening(self, widget):
         """ Prepare to open the Photo Database, trying to find it and evaluating command line argument """
@@ -247,23 +267,26 @@ class GPixPod:
 	mountpoint = self.filechooser.get_filename()
 	self.filechooser.destroy()
 	filename = os.path.join(mountpoint, 'Photos', 'Photo Database')
-	self.DBOpen(filename)
-        self.DB.dbcopyfullres = self.prefs['photo_copyfullres']
+        if os.path.isfile(filename):
+            if self.Ask(_('A Photo Database already exists'), _('Do you want to overwrite it completely?')):
+                os.remove(filename)
+        if not os.path.isfile(filename):
+	    self.DBOpen(filename)
+            self.DB.dbcopyfullres = self.prefs['photo_copyfullres']
 
     def CheckIpodModel(self, photodb_filename):
         """ Check for a supported iPod model """
 	ipod_mountpoint = photodb_filename.replace(os.path.join('', 'Photos', 'Photo Database'), '') 
 	self.ipod_model = getIpodModel(ipod_mountpoint)
-	if self.ipod_model == '5G' or self.ipod_model == 'Photo':
+	if self.ipod_model == '5G' or self.ipod_model == 'Photo' or self.ipod_model == 'Color':
 	    pass
         elif self.ipod_model == 'Nano':
             self.win.get_widget('expander1label').set_markup(_('<b>Fullscreen iPod image</b>'))
-            self.win.get_widget('expander2label').set_markup(_('<b>Small iPod thumbnail</b>'))
+            self.win.get_widget('expander2label').set_markup(_('<b>Small iPod thumbnail</b>'))  
             self.win.get_widget('expander3').hide()
             self.win.get_widget('expander4').hide()
-	elif self.ipod_model == 'Color':
-	    self.Say(_('You are using an <b>iPod %s</b>. Your model is considered as an iPod Photo and nothing is guaranteed to work.') % self.ipod_model,
-	             _('<b>Create backups before.</b> You have been warned! And please, report your feedback!'), gtk.MESSAGE_WARNING)
+            self.win.get_widget('save1thumb').hide()
+            self.win.get_widget('save3thumb').hide()
         else:
 	    self.Say(_('You are using an <b>iPod %s</b>. Please <b>quit now</b>: your model is not supported, and it never will be.') % self.ipod_model,
 	             _('It does not have a color screen and it does not support photos!'), gtk.MESSAGE_ERROR)
@@ -584,10 +607,10 @@ class GPixPod:
             syncf = open(self.syncfn, 'w+')
             cPickle.dump(self.sync, syncf)
             syncf.close()
+            self.ToggleProgress()
             if reopen:
                 self.DBOpen(self.DB.dbname)  # Re-opening the Photo Database to let user eventually continue work
        	        self.SetSensitive(('treeview1', 'open1', 'openbutton', 'addalbum1', 'addalbumbutton', 'cleancache1', 'rescue_database1', 'new_album_from_unreferenced_photos1'), True)
-	    self.ToggleProgress()
 	    while gtk.events_pending():
 	        gtk.main_iteration()
 	    self.work_left = False
@@ -619,7 +642,8 @@ class GPixPod:
         self.cellpx = gtk.CellRendererPixbuf()
         #self.cell2.set_property('stock-id', gtk.STOCK_ABOUT);
         self.columnpx = gtk.TreeViewColumn(_('Previews'), self.cellpx, pixbuf=0)
-	self.treeview.append_column(self.columnpx)
+        if self.win.get_widget('list_previews1').get_active():
+	    self.treeview.append_column(self.columnpx)
         self.treeview.append_column(self.column)
 	# Enable Drag & Drop (DND) for the treeview
 	targets = [('text/plain', 1, 1),
@@ -642,7 +666,10 @@ class GPixPod:
             albumpx = None
         a = 0
         for album in self.albums[1:]:
-            self.tree.set_value(self.tree.get_iter((a,)), 0, albumpx)
+            try:
+                self.tree.set_value(self.tree.get_iter((a,)), 0, albumpx)
+            except:
+                pass
             p = 0
             for pic in album.pics:
                 convthumb = os.path.join(self.DB.dbdir, '.ConvertedThumbs', '%sx%s' % (self.DB.thumbsizes[-1][0], self.DB.thumbsizes[-1][1]), '%s.png' % str(pic))
@@ -652,7 +679,10 @@ class GPixPod:
                     thumbcon = thumbf.read()
                     thumbf.close()
                     imgconvert.fromRGB565(thumbcon, self.DB.thumbsizes[-1][0], self.DB.thumbsizes[-1][1], True, False, convthumb)
-                self.tree.set_value(self.tree.get_iter((a, p)), 0, gtk.gdk.pixbuf_new_from_file(convthumb))
+                try:
+                    self.tree.set_value(self.tree.get_iter((a, p)), 0, gtk.gdk.pixbuf_new_from_file(convthumb))
+                except:
+                    pass
                 while gtk.events_pending():
                     gtk.main_iteration()
                 p += 1
@@ -662,10 +692,16 @@ class GPixPod:
         """ Destroy left-side list thumbnails previews icons """
         a = 0
         for album in self.albums[1:]:
-            self.tree.set_value(self.tree.get_iter((a,)), 0, None)
+            try:
+                self.tree.set_value(self.tree.get_iter((a,)), 0, None)
+            except:
+                pass
             p = 0
             for pic in album.pics:
-                self.tree.set_value(self.tree.get_iter((a, p)), 0, None)
+                try:
+                    self.tree.set_value(self.tree.get_iter((a, p)), 0, None)
+                except:
+                    pass
                 while gtk.events_pending():
                     gtk.main_iteration()
                 p += 1
@@ -700,67 +736,98 @@ class GPixPod:
         else:
             self.win.get_widget('notebook1').hide()
 
+    def DBPath2RealPath(self, dbpath):
+        """ Convert a Photo Database path (fullres) to real path """
+        realpath = u"%s" % dbpath
+        realpath = realpath.encode('utf-8')
+        realpath = os.path.join(self.DB.dbdir, *realpath.split(':')[1:])
+        return realpath
+
     def Details(self, widget): #, path, column):
         """ React to single-click on the tree view, displaying the details if a photo is selected. """
 	self.ShowActions(None)
         self.win.get_widget('remove_full_resolution').set_sensitive(False)
         self.win.get_widget('add_full_resolution').set_sensitive(False)
         self.win.get_widget('delete_reference1').set_sensitive(False)
+        self.win.get_widget('rename_album1').set_sensitive(False)
+        self.SetSensitive(('savefullres', 'save1thumb', 'save2thumb', 'save3thumb', 'save4thumb'), False)
         item_selected = self.treeview.get_selection().get_selected()[1]
         path = self.tree.get_path(item_selected)
         if self.win.get_widget('details_pane1').get_active():
             self.ClearDetails()
-            if len(path) > 1:  # We are interacting with an image.
-                self.SetSensitive(('expander1', 'expander2', 'expander3', 'expander4'), True)
+        if len(path) > 1:  # We are interacting with an image.
+            if self.win.get_widget('details_pane1').get_active():
                 self.imagehbox.show()
-	        album = None
-	        if len(self.albums) > (path[0]+1):
-	            album = self.albums[path[0]+1]
-	        if album != None and len(album.pics) > path[1] and self.photos_to_add.count(album.pics[path[1]]) == 0 and self.photos_to_delete.count(album.pics[path[1]]) == 0:
-	            # Not added now, the image was already there; neither deleted.
-                    self.current_photo_id = album.pics[path[1]]
-                    self.win.get_widget('delete_reference1').set_sensitive(True)  # Do deleting reference possible, since image is already stored.
-                    if self.DB.HasFullResolution(self.current_photo_id):
+                if self.ipod_model == 'Nano':
+                    self.SetSensitive(('expander1', 'expander2'), True)
+                else:
+                    self.SetSensitive(('expander1', 'expander2', 'expander3', 'expander4'), True)
+            if self.ipod_model == 'Nano':
+                self.SetSensitive(('save1thumb', 'save2thumb'), True)
+            else:
+                self.SetSensitive(('save1thumb', 'save2thumb', 'save3thumb', 'save4thumb'), True)
+	    album = None
+	    if len(self.albums) > (path[0]+1):
+	        album = self.albums[path[0]+1]
+	    if album != None and len(album.pics) > path[1] and self.photos_to_add.count(album.pics[path[1]]) == 0 and self.photos_to_delete.count(album.pics[path[1]]) == 0:
+	        # Not added now, the image was already there; neither deleted.
+                self.current_photo_id = album.pics[path[1]]
+                self.win.get_widget('delete_reference1').set_sensitive(True)  # Do deleting reference possible, since image is already stored.
+                if self.DB.HasFullResolution(self.current_photo_id):
+                    self.win.get_widget('savefullres').set_sensitive(True)
+                    if self.win.get_widget('details_pane1').get_active():
                         self.notebook.set_current_page(1)
 	                #fullres = '%s' % self.photos[album.pics[path[1]]-100].fullres.replace('\000', '')  # Removing null characters
                         fullres = u"%s" % self.photos[album.pics[path[1]]-100].fullres
                         fullres = fullres.encode('utf-8')
 	                photofile = os.path.join(self.DB.dbdir, *fullres.split(':')[1:])  # Converting from iPod (Mac) relative path to absolute native path
 	                self.ShowImage(photofile, path[0]+1, album.pics[path[1]])
-                        if self.fullres_to_delete.count(self.current_photo_id) == 0:
-	                    self.win.get_widget('remove_full_resolution').set_sensitive(True)
-                    else:
+                    if self.fullres_to_delete.count(self.current_photo_id) == 0:
+	                self.win.get_widget('remove_full_resolution').set_sensitive(True)
+                else:
+                    if self.win.get_widget('details_pane1').get_active():
                         self.label.set_markup(_('<i>No full resolution image</i>'))
                         self.notebook.set_current_page(2)
                         self.win.get_widget('label4').set_sensitive(False)
                         self.win.get_widget('expander1').emit('activate')
-                        fullres_add_sensitive = True
-                        for fullres_addition in self.fullres_to_add:
-                            if fullres_addition[1] == self.current_photo_id:
-	                        fullres_add_sensitive = False
-                                break
-                        self.win.get_widget('add_full_resolution').set_sensitive(fullres_add_sensitive)
-                else:  # The image has been just added
-		    itemtext = self.tree[(path[0], path[1])][1]
-		    itemdot = itemtext.find('.')
-		    for new_photo in self.photos_to_add:
-		        if new_photo[1] == (path[0]+1) and os.path.basename(new_photo[0]) == itemtext[itemdot+2:]:
-		            self.ShowImage(new_photo[0], path[0]+1, itemtext[:itemdot], False)
-		            self.current_photo_id = None  # Thumbnails do not exist yet
-			    break
-		    #self.label.set_text(_('Image has been just added. Its details will be available after saved.'))
-	    elif len(path) == 1:  # We are interacting with an album.
-                self.notebook.set_current_page(0)
-                self.ShowAlbum(path[0]+1)
-        if len(path) > 1:
-            self.win.get_widget('rename_album1').set_sensitive(False)
-        elif len(path) == 1:
+                    fullres_add_sensitive = True
+                    for fullres_addition in self.fullres_to_add:
+                        if fullres_addition[1] == self.current_photo_id:
+	                    fullres_add_sensitive = False
+                            break
+                    self.win.get_widget('add_full_resolution').set_sensitive(fullres_add_sensitive)
+            else:  # The image has been just added
+                itemtext = self.tree[(path[0], path[1])][1]
+	        itemdot = itemtext.find('.')
+	        for new_photo in self.photos_to_add:
+	            if new_photo[1] == (path[0]+1) and os.path.basename(new_photo[0]) == itemtext[itemdot+2:]:
+                        if self.win.get_widget('details_pane1').get_active():
+	                    self.ShowImage(new_photo[0], path[0]+1, itemtext[:itemdot], False)
+	                self.current_photo_id = None  # Thumbnails do not exist yet
+	                break
+		#self.label.set_text(_('Image has been just added. Its details will be available after saved.'))
+	elif len(path) == 1:  # We are interacting with an album.
             self.win.get_widget('rename_album1').set_sensitive(True)
             self.win.get_widget('delete_reference1').set_sensitive(True)
-	    if self.treeview.row_expanded(path):  # The album row is already expanded, so we collapse it.
-	        self.treeview.collapse_row(path)
-            else:
-	        self.treeview.expand_to_path(path)
+            if self.win.get_widget('details_pane1').get_active():
+                self.notebook.set_current_page(0)
+                self.ShowAlbum(path[0]+1)
+
+    def ToggleAlbum(self, widget):
+        """ Expand or collapse an album """
+        path = self.GetCurrentTVPath()
+        if self.treeview.row_expanded(path):  # The album row is already expanded, so we collapse it.
+            self.treeview.collapse_row(path)
+        else:
+            self.treeview.expand_to_path(path)
+
+    def ExpandPath(self, widget, path):
+        """ Expand treeview path """
+        self.treeview.expand_to_path(path)
+
+    def CollapsePath(self, widget, path):
+        """ Collapse treeview path """
+        self.treeview.collapse_row(path)
 
     def ShowAlbum(self, album_id):
         """ Show album details in the proper (first) notebook page """
@@ -829,7 +896,78 @@ class GPixPod:
     def DetailsDoubleClick(self, widget, path, column):
         """ React to double-click on the tree view, asking to rename if a photo album is selected. """
 	if len(path) == 1:  # The double-click occurred on an album
-            self.ShowRenameAlbum(None)
+            self.ToggleAlbum(None)
+        elif len(path) > 1:
+            self.ShowFullScreen(None)
+
+    def ShowFullScreen(self, widget, startslideshow=False, slideshowinterval=3):
+        """ Show the currently selected photo full screen """
+        try:
+            photoid = self.GetCurrentTVPhotoID()
+            self.slideshow_current_photoid = photoid
+            self.slideshow = gtk.glade.XML('gpixpod.glade', 'slideshowwindow', 'gpixpod')
+            self.slideshow_callbacks = {'on_slideshowclosebutton_clicked':self.SlideShowQuit,
+                                        'on_slideshowbackbutton_clicked':self.SlideShowBack,
+                                        'on_slideshowforwardbutton_clicked':self.SlideShowForward}
+            self.slideshow.signal_autoconnect(self.slideshow_callbacks)
+            self.slideshowwin = self.slideshow.get_widget('slideshowwindow')
+            self.slideshow.get_widget('slideshowviewport').modify_bg(gtk.STATE_NORMAL, gtk.gdk.Color())
+            self.slideshowwin.fullscreen()
+            self.SlideShowImage(photoid)
+            if startslideshow:
+                self.SlideShowT = gobject.timeout_add(slideshowinterval*1000, self.SlideShowStart)
+            while gtk.events_pending():
+                gtk.main_iteration()
+        except:
+            pass
+
+    def SlideShowImage(self, photoid):
+        """ Display the image in the slideshow window """
+        gdkwin = self.slideshowwin.window
+        gdkwin.set_cursor(gtk.gdk.Cursor(gtk.gdk.WATCH))
+        if photoid > 100:
+            self.slideshow.get_widget('slideshowbackbutton').set_sensitive(True)
+        else:
+            self.slideshow.get_widget('slideshowbackbutton').set_sensitive(False)
+        if photoid-99 < len(self.photos):
+            self.slideshow.get_widget('slideshowforwardbutton').set_sensitive(True)
+        else:
+            self.slideshow.get_widget('slideshowforwardbutton').set_sensitive(False)
+        if self.DB.HasFullResolution(photoid):
+            photo = self.DBPath2RealPath(self.photos[photoid-100].fullres)
+        else:
+            photo = self.ConvertThumbnail(photoid, 0)
+        toolbarh = self.slideshow.get_widget('toolbar3').size_request()[1]
+        pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(photo, gtk.gdk.screen_width(), gtk.gdk.screen_height()-toolbarh-5)
+        self.slideshow.get_widget('slideshowimage').set_from_pixbuf(pixbuf)
+        gdkwin.set_cursor(None)
+
+    def SlideShowBack(self, widget):
+        """ Display previous photo """
+        self.slideshow_current_photoid -= 1
+        self.SlideShowImage(self.slideshow_current_photoid)
+
+    def SlideShowForward(self, widget):
+        """ Display next photo """
+        self.slideshow_current_photoid += 1
+        self.SlideShowImage(self.slideshow_current_photoid)
+
+    def SlideShowQuit(self, widget):
+        """ Close the slideshow window """
+        self.slideshowwin.destroy()
+        try:
+            gobject.source_remove(self.SlideShowT)
+        except:
+            pass
+
+    def SlideShowStart(self):
+        """ Start automatic slideshow """
+        if self.slideshow_current_photoid-99 < len(self.photos):
+            self.SlideShowForward(None)
+            return True
+        else:
+            return False
+            #gobject.source_remove(self.SlideShowT)
 
     def ShowRenameAlbum(self, widget):
         """ Show the dialog to rename a photo album """
@@ -872,32 +1010,21 @@ class GPixPod:
 #                    self.win.get_widget('imageipodthumb%s' % thumbx).set_from_file(convthumbs[thumbx])
 #                self.current_photo_id = None
                    
-    def ExpandThumbnail(self, widget, thumb_index):
-        """ Expand and display the thumbnail with the specified index in the second page of the right pane """
-        # FIXME: Image is set every time, a way should be found to avoid setting again if already set
-        expander = self.win.get_widget('expander%s' % (thumb_index+1))
-        gdkwin = widget.window
-        if expander.get_expanded() == False and self.current_photo_id != None and self.current_photo_id > 0:
-            gdkwin.set_cursor(gtk.gdk.Cursor(gtk.gdk.WATCH))
-            thumb = os.path.join(self.DB.sglthumbsdir, '%sx%s' % (self.DB.thumbsizes[thumb_index][0], self.DB.thumbsizes[thumb_index][1]), str(self.current_photo_id))
+    def ConvertThumbnail(self, photo_id, thumb_index):
+        """ Convert the thumbnail of the specified index for the photo ID """
+        if photo_id > 0:
+            thumb = os.path.join(self.DB.sglthumbsdir, '%sx%s' % (self.DB.thumbsizes[thumb_index][0], self.DB.thumbsizes[thumb_index][1]), str(photo_id))
             convthumb = os.path.join(self.DB.dbdir, '.ConvertedThumbs', '%sx%s' % (self.DB.thumbsizes[thumb_index][0], self.DB.thumbsizes[thumb_index][1]), '%s.png' % 
-                                     str(self.current_photo_id))
+                                     str(photo_id))
             thumbf = open(thumb)
             thumbcon = thumbf.read()
             thumbf.close()
             if thumb_index == 0 and self.ipod_model != 'Nano':
                 if not os.path.isfile(convthumb):
-                    while gtk.events_pending():
-                        gtk.main_iteration()
                     #fromInterlacedUYVY(thumb, convthumb)
                     imgconvert.fromInterlacedUYVY(thumbcon, convthumb)
-                    while gtk.events_pending():
-                        gtk.main_iteration()
-                self.imageipodthumb0.set_from_file(convthumb)
             else:
                 if not os.path.isfile(convthumb):
-                    while gtk.events_pending():
-                        gtk.main_iteration()
                     if thumb_index == 0 and self.ipod_model == 'Nano':
                         #fromRGB565(thumb, self.DB.thumbsizes[thumb_index][0], self.DB.thumbsizes[thumb_index][1], False, savefilename=convthumb)
                         imgconvert.fromRGB565(thumbcon, self.DB.thumbsizes[thumb_index][0], self.DB.thumbsizes[thumb_index][1], False, True, convthumb)
@@ -907,9 +1034,18 @@ class GPixPod:
                     else:
                         #fromRGB565(thumb, self.DB.thumbsizes[thumb_index][0], self.DB.thumbsizes[thumb_index][1], savefilename=convthumb)
                         imgconvert.fromRGB565(thumbcon, self.DB.thumbsizes[thumb_index][0], self.DB.thumbsizes[thumb_index][1], True, False, convthumb)
-                    while gtk.events_pending():
-                        gtk.main_iteration()
-                self.win.get_widget('imageipodthumb%s' % thumb_index).set_from_file(convthumb)
+            return convthumb
+
+    def ExpandThumbnail(self, widget, thumb_index):
+        """ Expand and display the thumbnail with the specified index in the second page of the right pane """
+        expander = self.win.get_widget('expander%s' % (thumb_index+1))
+        gdkwin = widget.window
+        if expander.get_expanded() == False and self.current_photo_id != None and self.current_photo_id > 0:
+            gdkwin.set_cursor(gtk.gdk.Cursor(gtk.gdk.WATCH))
+            convthumb = self.ConvertThumbnail(self.current_photo_id, thumb_index)
+            while gtk.events_pending():
+                gtk.main_iteration()
+            self.win.get_widget('imageipodthumb%s' % thumb_index).set_from_file(convthumb)
             gdkwin.set_cursor(None)
 
     def CollapseThumbnails(self):
@@ -923,6 +1059,26 @@ class GPixPod:
         self.imageipodthumb2.set_from_file(None)
         self.imageipodthumb3.set_from_file(None)
  
+    def SaveThumb(self, widget, thumb_index):
+        """ Prompt to save the thumb with the specified index for the currently selected photo """
+        if self.ipod_model == 'Nano':
+            if thumb_index == 1:
+                thumb_index = 0
+            elif thumb_index == 3:
+                thumb_index = 1
+        if self.current_photo_id != None:
+            self.savethumbindex = thumb_index
+            savefilename = '%s-%sx%s.png' % (self.current_photo_id, self.DB.thumbsizes[thumb_index][0], self.DB.thumbsizes[thumb_index][1])
+            self.ShowChooser(None, self.ActuallySaveThumb, _('Images'), 'PIXBUF', False, self.prefs['path_lastsavedimages'],
+                             False, gtk.FILE_CHOOSER_ACTION_SAVE, _('Save image'), savefilename=savefilename)
+        
+    def ActuallySaveThumb(self, widget):
+        """ Actually save the thumb as triggered by the SaveThumb method """
+        thumbsavefn = self.filechooser.get_filename()
+        thumb = self.ConvertThumbnail(self.current_photo_id, self.savethumbindex)
+        shutil.copyfile(thumb, thumbsavefn)
+        self.filechooser.destroy()
+
     def Eject(self, widget):
         """ Eject the connected detected iPod """
         ejecting_i, ejecting_o = os.popen4('eject %s' % self.ipod_mountpoint)
@@ -937,7 +1093,7 @@ class GPixPod:
     def Quit(self, widget):
         """ Manage quitting, asking to save if there are unsaved changes. """
         if self.win.get_widget('save1').get_property('sensitive'):
-	    if self.Ask(_('There are changes <b>not</b> saved.'), _('Do you want to save before quit?')):
+	    if self.Ask(_('There are unsaved changes'), _('Do you want to save your changes before closing this program?'), gtk.BUTTONS_YES_NO):
 	        self.Save(None)
 	self.WritePreferences()
         if self.DB != None:
@@ -974,7 +1130,7 @@ class GPixPod:
         else:
             self.AddPhoto(None, None, False)
 
-    def ShowChooser(self, widget, ok_function, filter_name=None, pattern=None, multiple=False, path=None, preview=False, chooser_type=None, title=None):
+    def ShowChooser(self, widget, ok_function, filter_name=None, pattern=None, multiple=False, path=None, preview=False, chooser_type=None, title=None, savefilename=None):
         """ Show file chooser, to select and open file based on the pattern specified, passing to the specified function """
         self.chooser_callbacks = {'on_button1_clicked':self.DestroyChooser, 'on_button2_clicked':ok_function}
         self.chooser = gtk.glade.XML('gpixpod.glade', 'filechooserdialog1', 'gpixpod')
@@ -1020,7 +1176,9 @@ class GPixPod:
 	    self.filechooser.set_action(chooser_type)
 	if chooser_type == gtk.FILE_CHOOSER_ACTION_SAVE:
 	    self.filechooser.set_do_overwrite_confirmation(True)
-	    if self.savefilename != None:
+            if savefilename != None:
+                self.filechooser.set_current_name(savefilename)
+	    elif self.savefilename != None:
 	        self.filechooser.set_current_name(self.savefilename)
 	if title != None:
 	    self.filechooser.set_title(title)
@@ -1074,7 +1232,7 @@ class GPixPod:
 	self.albums_to_add.append(album_name)  # Mark the new album to be added when saving
 	#self.GetDBLists()
         if self.photoalbumicon and self.win.get_widget('list_previews1').get_active():
-            albumpx = gtk.gdk.pixbuf_new_from_file(self.photoalbumicon)
+            albumpx = gtk.gdk.pixbuf_new_from_file_at_size(self.photoalbumicon, 48, 48)
         else:
             albumpx = None
 	self.tree.append(None, [albumpx, '<b>%s</b>' % album_name])  # Adding the new album to the tree view
@@ -1112,6 +1270,10 @@ class GPixPod:
 	album_selected = self.tree.get_iter((path[0],))  # The parent album is obtained from the first element of the path
 	if filenames == None:
 	    filenames = self.filechooser.get_filenames()
+            if len(filenames) == 0:
+                filenames_folder = self.filechooser.get_current_folder()
+                if filenames_folder:
+                    filenames = [filenames_folder]
             behaviour = self.GetImageBehaviour()
             if self.chooser.get_widget('imagebehaviour_radiobutton2').get_active():
                 behaviour = 'Zoom'
@@ -1271,6 +1433,7 @@ class GPixPod:
 	    self.ToSave()  # Marking as unsaved
 	self.ClearDetails()  # We clear the image details (right pane)
 	self.HideActions(None)  # After deleting, we lose the selection: thus we should at least hide the related actions.
+        self.SetSensitive(('savefullres', 'save1thumb', 'save2thumb', 'save3thumb', 'save4thumb'), False)
 
 # not working yet and to delete
 #    def TreeviewDrag(self, treeview, context, selection, target_id, etime):
@@ -1314,23 +1477,42 @@ class GPixPod:
 #            context.finish(True, True, etime)
 #        return
 
+    def TreeviewClick(self, treeview, event):
+        """ React to mouse (right) button click on the treeview """
+        if event.button == 3:
+            pathinfo = treeview.get_path_at_pos(int(event.x), int(event.y))
+            if pathinfo:
+                path = pathinfo[0]
+                menu = gtk.Menu()
+                if len(path) == 1:  # Right click is happened on an album
+                    if self.treeview.row_expanded(path):
+                        collapseitem = gtk.MenuItem(_("Collapse"))
+                        collapseitem.connect("activate", self.CollapsePath, path)
+                        menu.append(collapseitem)                  
+                    else:
+                        expanditem = gtk.MenuItem(_("Expand"))
+                        expanditem.connect("activate", self.ExpandPath, path)
+                        menu.append(expanditem)
+                    renameitem = gtk.MenuItem(_("Rename"))
+                    renameitem.connect("activate", self.ShowRenameAlbum)
+                    menu.append(renameitem)
+                deleteitem = gtk.ImageMenuItem(gtk.STOCK_DELETE)
+                deleteitem.connect("activate", self.Delete)
+                menu.append(deleteitem)
+                menu.show_all()
+                menu.popup(None, None, None, event.button, event.time)
+
     def ShowActions(self, widget):
         """ Show "Add Photo" and "Delete" actions (to be called when a treeview item is selected) """
-        self.win.get_widget('addphoto1').set_sensitive(True)
-	self.win.get_widget('addphotobutton').set_sensitive(True)
-	self.win.get_widget('delete1').set_sensitive(True)
-	self.win.get_widget('deletebutton').set_sensitive(True)
-        self.win.get_widget('add_photo_reference1').set_sensitive(True)
-        self.win.get_widget('delete_reference1').set_sensitive(True)
+        self.SetSensitive(('addphoto1', 'addphotobutton', 'delete1', 'deletebutton', 'add_photo_reference1', 
+                          'delete_reference1', 'add_full_resolution', 'remove_full_resolution', 
+                          'showfullscreen', 'startslideshow'), True)
         
     def HideActions(self, widget):
         """ Hide "Add Photo" and "Delete" actions (to be called when no treeview items are selected) """
-        self.win.get_widget('addphoto1').set_sensitive(False)
-	self.win.get_widget('addphotobutton').set_sensitive(False)
-	self.win.get_widget('delete1').set_sensitive(False)
-	self.win.get_widget('deletebutton').set_sensitive(False)
-        self.win.get_widget('add_photo_reference1').set_sensitive(False)
-        self.win.get_widget('delete_reference1').set_sensitive(False)
+        self.SetSensitive(('addphoto1', 'addphotobutton', 'delete1', 'deletebutton', 'add_photo_reference1', 
+                           'delete_reference1', 'add_full_resolution', 'remove_full_resolution', 
+                           'showfullscreen', 'startslideshow'), False)
 
     def NewAlbumFromUnreferencedPhotos(self, widget):
         """ Generate a new photo album containing all the currently unreferenced (orphaned) photos """
@@ -1530,9 +1712,9 @@ class GPixPod:
             s = _('<b>Full resolution copies of your photos will not be uploaded on your iPod</b>')
         self.Say(s, _('This is now the default action of drag &amp; drop and of the <i>Add Photo</i> button. You can always use the <i>Add Photo with/without full resolution</i> buttons to override this setting'))
 
-    def Ask(self, sentence, secondary=None):
+    def Ask(self, sentence, secondary=None, buttons=gtk.BUTTONS_OK_CANCEL):
         """ Pop-up a question message dialog """
-        self.questionbox = gtk.MessageDialog(self.window, 0, gtk.MESSAGE_QUESTION, gtk.BUTTONS_OK_CANCEL)
+        self.questionbox = gtk.MessageDialog(self.window, gtk.DIALOG_MODAL, gtk.MESSAGE_QUESTION, buttons)
         if self.icon:
             self.questionbox.set_icon_from_file(self.icon)
 	self.questionbox.set_markup(sentence)
