@@ -120,6 +120,22 @@ class MH:
         self.PX_ADDED = []
         self.PX_REMOVED = []
  
+    def _PhotoCreationDate(self, filename):
+        """ Try to get the date of the image from the EXIF headers, otherwise from the file """
+	try:
+	    import mmpython.image.EXIF
+	    f = open(filename, 'rb')
+	    tags = mmpython.image.EXIF.process_file(f)
+	    f.close()
+	    # e.g. 'EXIF DateTimeOriginal' = '2004:12:15 16:49:49'
+	    if tags.has_key('EXIF DateTimeOriginal'):
+		return time.strptime(tags['EXIF DateTimeOriginal'].printable, "%Y:%m:%d %H:%M:%S")
+	except ImportError, e:
+	    pass
+        # Use the content modification time
+        st = os.stat(filename)
+	return time.localtime(st.st_mtime)
+
     def _FromScratchApply(self):
         """ Annul already opened Photo Database applying from scratch method """
         self._FromScratch()
@@ -301,6 +317,7 @@ class MH:
 	self._UpdateThumbsOffsets()
         self._ThumbsRebuild()
         self._UpdateThumbsIDs()
+        self._ThumbsChecksumStore()
 	self._SaveDB(filename)
 
     def Albums(self):
@@ -393,9 +410,10 @@ class MH:
         photos = self._ThumbsFilenames()
         chksum = []
         for ph in photos:
-             pf = open(ph)
-             chksum.append((ph, adler32(pf.read())))
-             pf.close()
+             #pf = open(ph)
+             #chksum.append((ph, adler32(pf.read())))
+             #pf.close()
+             chksum.append((ph, os.stat(ph)[8]))
         return dict(chksum)
 
     def _ThumbsChecksumStore(self):
@@ -416,6 +434,8 @@ class MH:
                 pgrs += 1
                 self.progress = pgrs/totnum
                 if chksum[t] != n:
+                    print chksum[t]
+                    print n
                     result = False
                     break
             self.progress = 1.0
@@ -495,8 +515,8 @@ class MH:
                 self.progress = pgrs/totnum
                 k.close()
             thmb.close()
-        self.thumbs_extracted = False
-        self._ThumbsChecksumStore()
+        #self.thumbs_extracted = False
+        #self._ThumbsChecksumStore()
         self.progress = 1.0
         #
 	#for t in range(0, len(self.thumbsizes)):
@@ -627,7 +647,7 @@ class MH:
         if photofullres == None:  # Then using DB global default
             photofullres = self.dbcopyfullres
 	name = os.path.basename(filename)  # check maximum filename string length FIXME
-	picdate = time.localtime()
+	picdate = self._PhotoCreationDate(filename)
 	albums = 0
 	last_mhii = None
         for x in range(0, len(self.dblines)):
@@ -666,7 +686,7 @@ class MH:
         self.PX_ADDED.append(photo_id)
         # Photo header (MHII)
         self.dblines.insert(mhpos, ['mhii', ['mhii', 152, 152+(len(self.thumbdbsizes)*180)+totfres, (freschild+len(self.thumbdbsizes)), photo_id, photo_id+albums, 
-                                             0, 0, 0, 0, appleTime(), appleTime(), 0], mhli])
+                                             0, 0, 0, 0, appleTime(picdate), appleTime(picdate), 0], mhli])
 	self.dblines[mhli][1][2] +=1  # Total number of pictures
 	self.dblines[0][1][7] += 1  # Incrementing next ID field in MHFD
 	# Inserting full resolution database details, if wanted
@@ -692,7 +712,7 @@ class MH:
     def _CopyFullResolution(self, filename):
         """ Copy the full resolution photo to destination """
 	name = os.path.basename(filename)
-	picdate = time.localtime()
+	picdate = self._PhotoCreationDate(filename)
 	dest_dir = os.path.join(self.dbdir, 'Full Resolution', *map(str, picdate[:3]))
 	if not os.path.isdir(dest_dir):
 	    os.makedirs(dest_dir)
@@ -778,7 +798,7 @@ class MH:
                     if self.dblines[x][0] == 'mhii' and self.dblines[x][1][4] == photo_id:
                         if self._HasFullResolution(x) == False:
                             name = os.path.basename(filename)
-                            picdate = time.localtime()
+                            picdate = self._PhotoCreationDate(filename)
               	            self._CopyFullResolution(filename)
 	                    fullresdb = (':Full Resolution:%s:%s:%s:%s' % (picdate[0], picdate[1], picdate[2], name)).encode('utf-16-le')
 	                    fullresdb_len = len(fullresdb)
